"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongooseHelper = void 0;
class MongooseHelper {
    static buildSelectAggregated(groupSelect = {}) {
        const objGroup = {};
        if (Object.keys(groupSelect).length > 0) {
            Object.keys(groupSelect).forEach((key) => {
                objGroup[key] = {
                    $first: `$${groupSelect[key]}`
                };
            });
        }
        return objGroup;
    }
    static buildRegexFilterQuery(objSearch = {}) {
        const objSearchRegex = {};
        Object.keys(objSearch).forEach(function (key) {
            if (objSearch[key] == null)
                return;
            objSearchRegex[key] = objSearch[key];
            if (typeof objSearch[key] === 'string')
                objSearchRegex[key] = new RegExp(`^${objSearch[key]}$`, 'i');
        });
        return objSearchRegex;
    }
    static buildLookupAggregate(from, joinFrom, joinLet, match = {}) {
        const letObj = {};
        letObj[`${joinFrom}`] = `\$${joinLet}`;
        return {
            $lookup: {
                from,
                let: letObj,
                pipeline: [
                    {
                        $match: {
                            $expr: {
                                $and: [
                                    { $eq: [`\$${joinFrom}`, `\$\$${joinFrom}`] },
                                    ...MongooseHelper.BuildMatchArrayLookup(match)
                                ]
                            }
                        }
                    },
                    {
                        $limit: 1
                    }
                ],
                as: 'aggElement'
            }
        };
    }
    static BuildMatchArrayLookup(match = {}) {
        if (Object.keys(match).length === 0)
            return [];
        const arrLookup = [];
        const keys = Object.keys(match);
        const values = Object.values(match);
        for (let i = 0; i < keys.length; i++) {
            const arrObj = [];
            arrObj.push(`\$${keys[i]}`);
            arrObj.push(values[i]);
            arrLookup.push({
                $eq: arrObj
            });
        }
        return arrLookup;
    }
}
exports.MongooseHelper = MongooseHelper;
