"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MongooseRepository = void 0;
const common_1 = require("@nestjs/common");
const mongodb__exception_1 = require("../core/error-handling/exception/mongodb-.exception");
const mongoose_helper_1 = require("./mongoose.helper");
const mongoose_1 = require("@nestjs/mongoose");
class MongooseRepository {
    constructor(model) {
        this.model = model;
        this.logger = new common_1.Logger(this.constructor.name);
    }
    insertOne(item, name) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.create(item).then((savedDoc) => {
                this.logger.log(`${item.constructor.name} '${name}' saved successfully!`);
                this.logger.log(`ID: ${savedDoc._id}`);
                return savedDoc._id;
            }, (err) => {
                this.logger.error(err.message);
                throw new mongodb__exception_1.MongoDBException(err.message, err.code);
            });
        });
    }
    create(document) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                this.model.create(document, function (err, savedDoc) {
                    if (err)
                        reject(err);
                    resolve(savedDoc);
                });
            }));
        });
    }
    groupBy(group, match = {}, select = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregateParams = [];
            if (Object.keys(match).length > 0)
                aggregateParams.push({ $match: match });
            aggregateParams.push({
                $group: Object.assign({ _id: group, count: { $sum: 1 } }, mongoose_helper_1.MongooseHelper.buildSelectAggregated(select)),
            });
            return this.model.aggregate(aggregateParams);
        });
    }
    aggregate(group, match = {}, select = {}, unwind = '') {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregateParams = [];
            if (Object.keys(match).length > 0)
                aggregateParams.push({ $match: match });
            if (unwind.length > 0)
                aggregateParams.push({ $unwind: `\$${unwind}` });
            aggregateParams.push({
                $group: Object.assign(Object.assign(Object.assign({}, group), { count: { $sum: 1 } }), select),
            });
            return this.model.aggregate(aggregateParams);
        });
    }
    aggregateNotExists(from, joinFrom, joinLet, match = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const aggregateParams = [];
            aggregateParams.push(mongoose_helper_1.MongooseHelper.buildLookupAggregate(from, joinFrom, joinLet, match));
            const matchParam = {};
            matchParam[`aggElement.${joinFrom}`] = { $exists: false };
            if (Object.keys(match).length > 0)
                aggregateParams.push({ $match: Object.assign(Object.assign({}, matchParam), match) });
            aggregateParams.push({
                $project: {
                    likes: false,
                },
            });
            return this.model.aggregate(aggregateParams);
        });
    }
    findAll(select = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Object.keys(select).length === 0)
                select = { _id: 0 };
            return this.model.find({}).select(select).lean().exec();
        });
    }
    findById(id, select = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Object.keys(select).length === 0)
                select = { _id: 0 };
            return this.model.findById(id).select(select).lean().exec();
        });
    }
    updateOneById(id, data) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.updateOne({ _id: id }, data);
            this.logger.log(`${(0, mongoose_1.getModelToken)(this.model.name)} ${id} - Succesfully updated!.`);
            return res;
        });
    }
    updateOne(query, data, pushData = {}, strict = false) {
        return __awaiter(this, void 0, void 0, function* () {
            this.model.findOneAndUpdate(query, Object.assign({ $set: data }, pushData), { upsert: false, strict }, function (err) {
                if (err)
                    throw new mongodb__exception_1.MongoDBException(err.message, err.code);
            });
        });
    }
    deleteOneById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.model.deleteOne({ id });
        });
    }
    find(searchParams, select = {}, sort = {}, regex = true) {
        return __awaiter(this, void 0, void 0, function* () {
            if (Object.keys(select).length === 0)
                select = { _id: 0 };
            searchParams = regex
                ? mongoose_helper_1.MongooseHelper.buildRegexFilterQuery(searchParams)
                : searchParams;
            let res = this.model.find(searchParams);
            if (typeof sort === 'object' && Object.keys(sort).length > 0)
                res = res.sort(sort);
            return res.select(select).lean().exec();
        });
    }
}
exports.MongooseRepository = MongooseRepository;
